#! /usr/bin/perl
#
# This file builds the core DAL support.
#
# $Id: Makefile.PL,v 1.7 2000/09/29 10:34:40 gvacanti Exp $
#

use Env;
use ExtUtils::MakeMaker;
use FileHandle;
use File::Basename;

WriteMakefile(
              'NAME'	=> 'DAL_C_lib',
              'OBJECT' => 'DAL_C_lib_wrap.o',
	      'CONFIGURE' => \&initialize,
              'LD' => '$(CC) -shared',

		# Digital Unix:
		#	For whatever reason the tmp.a created by GNU ar 
		#	(binutils 2.9.1) is corrupted.
		#	Therefore make sure that Digital Unix ar is used
		#		Achim Bohnet <ach@mpe.mpg.de>
		($^O eq 'dec_osf' ? ('dynamic_lib' => {'ARMAYBE' => '/usr/bin/ar'} ) : () )
);


sub initialize {
    my $self = {};
    local $\ = "\n";

    die "Have you defined SAS_DIR?\n" if not defined $SAS_DIR;

    my $fh = new FileHandle;
    $fh->open("../../VERSION", "r") or die $!;
    my $ver = <$fh>;
    chomp $ver;
    undef $fh;
    $self->{'VERSION'} = $ver;

    $self->{'INC'} = "-I$SAS_DIR/include -I$SAS_DIR/include/dalc -I$SAS_DIR/include/dal -I$SAS_DIR/include/utils";

###    $self->{'LIBS'} = ["-L$SAS_DIR/lib -ldal_c -ldal -lutils -lerror -lcfitsio -L/opt/gnu/lib/ -lstdc++ -lg++"];

    if($DECEIT){
	print "\nDAL_C_lib will be built for use with deceit\n";
	$self->{'LIBS'} = ["-L$SAS_DIR/lib -ldalc"];
    } else {
	print "\nDAL_C_lib will be built for stand alone use\n";
	
	my $os_libs = "";
	foreach ($^O) {
	    # DEC OSF needs libgcc
	    $_ eq 'dec_osf' and do {
		$os_libs = "-L" . dirname(`gcc -print-libgcc-file-name`) . " -lgcc";
		last;
	    };
	}

	my $libs = "-L$SAS_DIR/lib -ldal -lutils -lerror";
	$libs = $libs . " " . find_egcs_library('stdc++');
	$self->{'LIBS'} = [$libs];
    }
	
    return $self;
}

# -- these to be moved somewhere else

# find a header file in C_INCLUDE_PATH or CPLUS_INCLUDE_PATH
# return -I$dir 
sub find_header {
    my $header = shift;
    my @c_include_path = split /:/, $C_INCLUDE_PATH;
    my @cplus_include_path = split /:/, $CPLUS_INCLUDE_PATH;
    foreach my $dir (@c_include_path)
    {
	if(-e "$dir/$header")
	{
	    return "-I$dir";
	}
    }
    foreach my $dir (@cplus_include_path)
    {
	if(-e "$dir/$header")
	{
	    return "-I$dir";
	}
    }
    die("$header not found.\n");
}



# find a library in LD_LIBRARY_PATH or LIBRARY_PATH
# return -L$dir -l$lib
sub find_library {    
    my $lib = shift;
    my @ld_library_path = split /:/, $LD_LIBRARY_PATH;
    my @library_path = split /:/, $LIBRARY_PATH;
    foreach my $dir (@ld_library_path)
    {
	if(-e "$dir/lib$lib.so" or -e "$dir/lib$lib.a")
	{
	    return "-L$dir -l$lib";
	}
    }
    foreach my $dir (@library_path)
    {
	if(-e "$dir/lib$lib.a" or -e "$dir/lib$lib.so")
	{
	    return "-L$dir -l$lib";
	}
    }
    die("$lib not found.\n");
}

# find g++ library
# return -L$dir -l$lib
sub find_egcs_library {
    my $lib = shift;
    my $liba = `g++ -print-file-name=lib$lib.a`;
    my $libso = `g++ -print-file-name=lib$lib.so`;
    chomp($liba);
    chomp($libso);
    if(-e $liba)
    {
	my $l = quotemeta($lib);
	$liba =~ m{(/.+/)lib$l\.a};
	return "-L$1 -l$lib";
    } elsif(-e $libso)
    {
	my $l = quotemeta($lib);
	$libso =~ m{(/.+/)lib$l\.so};
	return "-L$1 -l$lib";
    } else {
	die "$lib not found.\n";
    }
}
